/**
 * @file charge.h
 * @author Tanguy Cavagna (tanguy.cavagna@etu.hesge.ch)
 * @brief Charge file header
 * @version 0.1
 * @date 2021-07-12
 * 
 * @copyright Copyright (c) 2021
 * 
 */
#ifndef _CHARGE_H_
#define _CHARGE_H_

#include <stdlib.h>
#include <stdbool.h>
#include "utils.h"
#include "vector/vector.h"
#include "gfx/gfx.h"

/**
 * @brief Compute E*qP/norm(qP)
 * 
 * @param c Parent charge
 * @param p Point on which to compute e
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false norm(qP) < eps
 */
bool compute_e(charge_t c, vec2 p, double eps, vec2 *e);

/**
 * @brief Compute the normilized sum of Ei*qiP/norm(qiP)
 * 
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param p Current computed point
 * @param eps Minimal gap between field and charge
 * @param e Electric field
 * @return bool false for some qiP, norm(qiP) < eps
 */
bool compute_total_normalized_e(charge_t *charges, int num_charges, vec2 p, 
                                double eps, vec2 *e);

/**
 * @brief Compute and then draw all the points belonging to a field line,
 *        starting from pos0
 * 
 * @param ctxt Context to draw on
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param dx Space between computed points on the field line
 * @param pos0 First post to start the computing of the field line
 * @param x0 Minal x of the universe
 * @param x1 Maximal x of the universe
 * @param y0 Minal y of the universe
 * @param y1 Maximal y of the universe
 * @return bool false if pos0 not a valid pos (e.g. too close to a charge)
 */
bool draw_field_line(struct gfx_context_t *ctxt, charge_t *charges, 
                            int num_charges, double dx, vec2 pos0, double x0, 
                            double x1, double y0, double y1);

/**
 * @brief Draw all charges
 *        A circle with a minus sign for negative charges
 *        A circle with a plus sign for positive charges
 * 
 * @param ctxt Context to draw on
 * @param charges All the charges
 * @param num_charges Number of charges in action
 * @param x0 Minal x of the universe
 * @param x1 Maximal x of the universe
 * @param y0 Minal y of the universe
 * @param y1 Maximal y of the universe
 */
void draw_charges(struct gfx_context_t *ctxt, charge_t *charges,
                         int num_charges, double x0, double x1, double y0, double y1);

/**
 * @brief Generate a given amount of points
 * 
 * @param points Points array
 * @param nb_points Number of point to generate
 */
void generate_points(vec2 points[], int nb_points);

#endif // _CHARGE_H_