/*! \file Library for vector operations
          All the operations are made with a Cartesian vector. If you want a Polar vector, call to_polar().
*/

// To be able to use mathematical constant defined in math.h
#ifndef __USE_MISC
#define __USE_MISC
#endif

#include <math.h>
#include <stdio.h>
#include "vector.h"

vec2 init_vector(double x, double y) {
    vec2 v = {x, y};
    
    return v;
}

/*! \fn vector add_vector(vector v1, vector v2)
    \brief Add two vector and return the final vector
    \param v1 First vector to add
    \param v2 Second vector to add
*/
vec2 add_vector(vec2 v1, vec2 v2) {
    vec2 resultVector;

    double finalX = v1.x + v2.x;
    double finalY = v1.y + v2.y;

    resultVector.x = finalX;
    resultVector.y = finalY;

    return resultVector;
}

/*! \fn vector sub_vector(vector v1, vector v2)
    \brief Substract two vector and return the final vector
    \param v1 First vector
    \param v2 Vector we want to substract to the first
*/
vec2 sub_vector(vec2 v1, vec2 v2) {
    // First we reverse the direction of the vector we want to substract
    v2.x *= -1;
    v2.y *= -1;

    // Then add them as usual
    return add_vector(v1, v2);    
}

/*! \fn vector mult_by_scalar(vector v1, int scalar)
    \brief Multiply a vector by a scalar
    \param v1 Vector to multiply
    \param scalar Scalar to use for the multiplication
*/
vec2 mult_by_scalar(vec2 v1, int scalar) {
    vec2 resultVector = {v1.x * scalar, v1.y * scalar};

    return resultVector;
}

/*! \fn double norm(vector v1)
    \brief Get the norm of a given vector
    \param v1 Vector to multiply
*/
double norm(vec2 v1) {
    double vectorNorm;

    vectorNorm = sqrt(pow(v1.x, 2) + pow(v1.y, 2));

    return vectorNorm;
}

/*! \fn double dot_multiply(vector v1, vector v2)
    \brief Multiply two vector with the dot product formula
    \param v1 First vector to multiply
    \param v2 Second vector to multiply
*/
double dot_multiply(vec2 v1, vec2 v2) {
    return (v1.x * v2.x) + (v1.y * v2.y);
}

/*! \fn polarVector to_polar(vector v1)
    \brief Convert a Cartesian vector to a Polar vector
    \param v1 Cartesian vector
*/
polarVector to_polar(vec2 v1) {
    polarVector p_vector;

    p_vector.f = norm(v1);
    p_vector.a = atan(v1.y / v1.x) * (180.0 / M_PI); // The atan return a radian angle. Use `* 180 / M_PI` to convert it in degree

    return p_vector;
}

/*! \fn vector to_cartesian(polarVector v1)
    \brief Convert a Polar vector to a Cartesian vector
    \param v1 Polar vector
*/
vec2 to_cartesian(polarVector v1) {
    vec2 c_vector;

    // The cos work with radian angle. Convert degree to radiant inside the cos function to have the correct force
    c_vector.x = v1.f * cos(v1.a * (M_PI / 180.0));
    c_vector.y = v1.f * sin(v1.a * (M_PI / 180.0));

    return c_vector;
}
